# sage_setup: distribution = sagemath-categories
"""
Laurent Series

EXAMPLES::

    sage: R.<t> = LaurentSeriesRing(GF(7), 't'); R
    Laurent Series Ring in t over Finite Field of size 7
    sage: f = 1/(1-t+O(t^10)); f
    1 + t + t^2 + t^3 + t^4 + t^5 + t^6 + t^7 + t^8 + t^9 + O(t^10)

Laurent series are immutable::

    sage: f[2]
    1
    sage: f[2] = 5
    Traceback (most recent call last):
    ...
    IndexError: Laurent series are immutable

We compute with a Laurent series over the complex mpfr numbers.

::

    sage: K.<q> = Frac(CC[['q']]); K                                                    # needs sage.rings.real_mpfr
    Laurent Series Ring in q over Complex Field with 53 bits of precision
    sage: q                                                                             # needs sage.rings.real_mpfr
    1.00000000000000*q

Saving and loading.

::

    sage: loads(q.dumps()) == q                                                         # needs sage.rings.real_mpfr
    True
    sage: loads(K.dumps()) == K                                                         # needs sage.rings.real_mpfr
    True

IMPLEMENTATION: Laurent series in Sage are represented internally
as a power of the variable times the unit part (which need not be a
unit - it's a polynomial with nonzero constant term). The zero
Laurent series has unit part 0.

AUTHORS:

- William Stein: original version

- David Joyner (2006-01-22): added examples

- Robert Bradshaw (2007-04): optimizations, shifting

- Robert Bradshaw: Cython version
"""
# ****************************************************************************
#       Copyright (C) 2005 William Stein <wstein@gmail.com>
#                     2017 Vincent Delecroix <20100.delecroix@gmail.com>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#
#    This code is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#    General Public License for more details.
#
#  The full text of the GPL is available at:
#
#                  https://www.gnu.org/licenses/
# ****************************************************************************

from sage.rings.infinity import infinity

from sage.rings.rational_field import QQ
import sage.misc.latex
from sage.rings.polynomial.laurent_polynomial import LaurentPolynomial_univariate
from sage.rings.power_series_ring_element cimport PowerSeries
from sage.structure.element cimport Element, AlgebraElement
from sage.structure.richcmp cimport richcmp_not_equal, rich_to_bool
from sage.misc.derivative import multi_derivative


def is_LaurentSeries(x):
    from sage.misc.superseded import deprecation_cython
    deprecation_cython(38266,
                       "The function is_LaurentSeries is deprecated; "
                       "use 'isinstance(..., LaurentSeries)' instead.")
    return isinstance(x, LaurentSeries)


cdef class LaurentSeries(AlgebraElement):
    r"""
    A Laurent Series.

    We consider a Laurent series of the form `t^n \cdot f` where `f` is a
    power series.

    INPUT:

    - ``parent`` -- a Laurent series ring

    - ``f`` -- a power series (or something can be coerced
      to one); note that ``f`` does *not* have to be a unit

    - ``n`` -- (default: 0) integer
    """
    def __init__(self, parent, f, n=0):
        r"""
        Initialize ``self``.

        OUTPUT: a Laurent series

        EXAMPLES::

            sage: R.<q> = LaurentSeriesRing(ZZ)
            sage: R([1,2,3])
            1 + 2*q + 3*q^2
            sage: R([1,2,3], -5)
            q^-5 + 2*q^-4 + 3*q^-3

        ::

            sage: # needs sage.rings.finite_rings sage.rings.padics
            sage: S.<s> = LaurentSeriesRing(GF(5))
            sage: T.<t> = PowerSeriesRing(pAdicRing(5))
            sage: S(t)
            s
            sage: parent(S(t))
            Laurent Series Ring in s over Finite Field of size 5
            sage: parent(S(t)[1])
            Finite Field of size 5
        """
        AlgebraElement.__init__(self, parent)

        if isinstance(f, LaurentSeries):
            n += (<LaurentSeries>f).__n
            if (<LaurentSeries>f).__u._parent is parent._power_series_ring:
                f = (<LaurentSeries>f).__u
            else:
                f = parent._power_series_ring((<LaurentSeries>f).__u)
        elif isinstance(f, LaurentPolynomial_univariate):
            f = f(parent.gen())
        elif isinstance(f, dict):
            ## Sanitize input to make sure all exponents are nonnegative,
            ## adjusting n to match.
            n1 = min(f.keys())
            if n1 < 0:
                f = {e - n1: c for e, c in f.items()}
                n += n1
            f = parent._power_series_ring(f)
        elif not isinstance(f, PowerSeries):
            f = parent._power_series_ring(f)
        ## now this is a power series, over a different ring ...
        ## requires that power series rings with same vars over the
        ## same parent are unique.
        elif parent is not f.parent():
            f = parent._power_series_ring(f)

        # self is that t^n * u:
        if not f:
            if n is infinity:
                self.__n = 0
                self.__u = parent._power_series_ring.zero()
            else:
                self.__n = n
                self.__u = f
        else:
            val = f.valuation()
            if val is infinity:
                self.__n = 0
                self.__u = f
            elif val == 0:
                self.__n = n    # power of the variable
                self.__u = f    # unit part
            else:
                self.__n = n + val
                self.__u = f >> val

    def __reduce__(self):
        return self._parent, (self.__u, self.__n)

    def change_ring(self, R):
        """
        Change the base ring of ``self``.

        EXAMPLES::

            sage: R.<q> = LaurentSeriesRing(ZZ)
            sage: p = R([1,2,3]); p
            1 + 2*q + 3*q^2
            sage: p.change_ring(GF(2))
            1 + q^2
        """
        return self._parent.change_ring(R)(self)

    def is_unit(self):
        """
        Return ``True`` if this is Laurent series is a unit in this ring.

        EXAMPLES::

            sage: R.<t> = LaurentSeriesRing(QQ)
            sage: (2 + t).is_unit()
            True
            sage: f = 2 + t^2 + O(t^10); f.is_unit()
            True
            sage: 1/f
            1/2 - 1/4*t^2 + 1/8*t^4 - 1/16*t^6 + 1/32*t^8 + O(t^10)
            sage: R(0).is_unit()
            False
            sage: R.<s> = LaurentSeriesRing(ZZ)
            sage: f = 2 + s^2 + O(s^10)
            sage: f.is_unit()
            False
            sage: 1/f
            Traceback (most recent call last):
            ...
            ValueError: constant term 2 is not a unit

        ALGORITHM: A Laurent series is a unit if and only if its "unit
        part" is a unit.
        """
        return self.__u.is_unit()

    def is_zero(self):
        """
        EXAMPLES::

            sage: x = Frac(QQ[['x']]).0
            sage: f = 1/x + x + x^2 + 3*x^4 + O(x^7)
            sage: f.is_zero()
            0
            sage: z = 0*f
            sage: z.is_zero()
            1
        """
        return self.__u.is_zero()

    def is_monomial(self):
        """
        Return ``True`` if this element is a monomial.  That is, if ``self`` is
        `x^n` for some integer `n`.

        EXAMPLES::

            sage: k.<z> = LaurentSeriesRing(QQ, 'z')
            sage: (30*z).is_monomial()
            False
            sage: k(1).is_monomial()
            True
            sage: (z+1).is_monomial()
            False
            sage: (z^-2909).is_monomial()
            True
            sage: (3*z^-2909).is_monomial()
            False
        """
        return self.__u.is_monomial()

    def __bool__(self):
        """
        EXAMPLES::

            sage: R.<t> = LaurentSeriesRing(ZZ)
            sage: bool(t)
            True
            sage: bool(1/t)
            True
            sage: bool(2 + t)
            True
            sage: bool(1/(1-t))
            True
            sage: bool(1 + O(t^3))
            True
            sage: bool(t + O(t^3))
            True
            sage: bool(O(t^3))
            False
            sage: bool(O(t^-3))
            False
            sage: bool(R.zero())
            False
        """
        return bool(self.__u)

    def _im_gens_(self, codomain, im_gens, base_map=None):
        """
        Return the image of this series under the map that sends the generators of
        the parent to ``im_gens``.

        EXAMPLES::

            sage: # needs sage.rings.number_field
            sage: Zx.<x> = ZZ[]
            sage: K.<i> = NumberField(x^2 + 1)
            sage: R.<t> = LaurentSeriesRing(K)
            sage: z = t^-1 + i*t
            sage: z._im_gens_(R, [t^2])
            t^-2 + i*t^2

        The argument ``base_map`` is not yet supported, because it isn't over power series::

            sage: cc = K.hom([i])                                                       # needs sage.rings.number_field
            sage: z._im_gens_(R, [t^2], base_map=cc)                                    # needs sage.rings.number_field
            Traceback (most recent call last):
            ...
            NotImplementedError
        """
        x = im_gens[0]
        return codomain(self.__u._im_gens_(codomain, im_gens, base_map=base_map) * x**self.__n)

    cdef _normalize(self):
        r"""
        A Laurent series is a pair (u(t), n), where either u=0 (to some
        precision) or u is a unit. This pair corresponds to
        `t^n\cdot u(t)`.
        """
        if self.is_zero():
            return
        v = self.__u.valuation()
        if v == 0:
            return
        self.__n += v
        self.__u = self.__u.valuation_zero_part()

    def _repr_(self):
        """
        EXAMPLES::

            sage: R.<t> = LaurentSeriesRing(QQ)
            sage: (2 + (2/3)*t^3).__repr__()
            '2 + 2/3*t^3'
        """
        if self.is_zero():
            if self.prec() is infinity:
                return "0"
            else:
                return "O(%s^%s)"%(self._parent.variable_name(),self.prec())
        s = " "
        v = self.__u.list()
        valuation = self.__n
        m = len(v)
        X = self._parent.variable_name()
        atomic_repr = self._parent.base_ring()._repr_option('element_is_atomic')
        first = True
        for n in range(m):
            x = v[n]
            e = n + valuation
            x = str(x)
            if x != '0':
                if not first:
                    s += " + "
                if not atomic_repr and (x[1:].find("+") != -1 or x[1:].find("-") != -1):
                    x = "(%s)"%x
                if e == 1:
                    var = "*%s"%X
                elif e == 0:
                    var = ""
                else:
                    var = "*%s^%s"%(X,e)
                s += "%s%s"%(x,var)
                first = False
        s = s.replace(" + -", " - ")
        s = s.replace(" 1*"," ")
        s = s.replace(" -1*", " -")
        if self.prec() == 0:
            bigoh = "O(1)"
        elif self.prec() == 1:
            bigoh = "O(%s)"%self._parent.variable_name()
        else:
            bigoh = "O(%s^%s)"%(self._parent.variable_name(),self.prec())
        if self.prec() != infinity:
            if s == " ":
                return bigoh
            s += " + %s"%bigoh
        return s[1:]

    def verschiebung(self, n):
        r"""
        Return the ``n``-th Verschiebung of ``self``.

        If `f = \sum a_m x^m` then this function returns `\sum a_m x^{mn}`.

        EXAMPLES::

            sage: R.<x> = LaurentSeriesRing(QQ)
            sage: f = -1/x + 1 + 2*x^2 + 5*x^5
            sage: f.V(2)
            -x^-2 + 1 + 2*x^4 + 5*x^10
            sage: f.V(-1)
            5*x^-5 + 2*x^-2 + 1 - x
            sage: h = f.add_bigoh(7)
            sage: h.V(2)
            -x^-2 + 1 + 2*x^4 + 5*x^10 + O(x^14)
            sage: h.V(-2)
            Traceback (most recent call last):
            ...
            ValueError: For finite precision only positive arguments allowed

        TESTS::

            sage: R.<x> = LaurentSeriesRing(QQ)
            sage: f = x
            sage: f.V(3)
            x^3
            sage: f.V(-3)
            x^-3
            sage: g = 2*x^(-1) + 3 + 5*x
            sage: g.V(-1)
            5*x^-1 + 3 + 2*x
        """
        if n == 0:
            raise ValueError('n must be nonzero')

        if n < 0:
            if not self.prec() is infinity:
                raise ValueError('For finite precision only positive arguments allowed')

            exponents = [e * n for e in self.exponents()]
            u = min(exponents)
            exponents = [e - u for e in exponents]
            coefficients = self.coefficients()
            zero = self.base_ring().zero()
            w = [zero] * (max(exponents) + 1)
            for i in range(len(exponents)):
                e = exponents[i]
                c = coefficients[i]
                w[e] = c
            l = LaurentSeries(self._parent, w, u)
        else:
            __u = self.__u.V(n)
            __n = <long>self.__n * n
            l = LaurentSeries(self._parent, __u, __n)
        return l

    V = verschiebung

    def _latex_(self):
        r"""
        EXAMPLES::

            sage: x = Frac(QQ[['x']]).0
            sage: f = (17/2)*x^-2 + x + x^2 + 3*x^4 + O(x^7)
            sage: latex(f)
            \frac{\frac{17}{2}}{x^{2}} + x + x^{2} + 3x^{4} + O(x^{7})

        Verify that :issue:`6656` has been fixed::

            sage: R.<a,b> = PolynomialRing(QQ)
            sage: T.<x> = LaurentSeriesRing(R)
            sage: y = a*x + b*x
            sage: y._latex_()
            '\\left(a + b\\right)x'
            sage: latex(y)
            \left(a + b\right)x
        """
        if self.is_zero():
            if self.prec() is infinity:
                return "0"
            else:
                return "0 + \\cdots"
        s = " "
        v = self.__u.list()
        valuation = self.__n
        m = len(v)
        X = self._parent.latex_variable_names()[0]
        atomic_repr = self._parent.base_ring()._repr_option('element_is_atomic')
        first = True
        for n in range(m):
            x = v[n]
            e = n + valuation
            x = sage.misc.latex.latex(x)
            if x != '0':
                if not first:
                    s += " + "
                if not atomic_repr and e > 0 and (x[1:].find("+") != -1 or x[1:].find("-") != -1):
                    x = "\\left(%s\\right)"%x
                if e == 1:
                    var = "|%s"%X
                elif e == 0:
                    var = ""
                elif e > 0:
                    var = "|%s^{%s}"%(X,e)
                if e >= 0:
                    s += "%s%s"%(x,var)
                else: # negative e
                    if e == -1:
                        s += "\\frac{%s}{%s}"%(x, X)
                    else:
                        s += "\\frac{%s}{%s^{%s}}"%(x, X,-e)
                first = False
        s = s.replace(" + -", " - ")
        s = s.replace(" 1|"," ")
        s = s.replace(" -1|", " -")
        s = s.replace("|","")
        pr = self.prec()
        if pr != infinity:
            if pr == 0:
                bigoh = "O(1)"
            elif pr == 1:
                bigoh = "O(%s)"%(X,)
            else:
                bigoh = "O(%s^{%s})"%(X,pr)
            if s == " ":
                return bigoh
            s += " + %s"%bigoh
        return s[1:]

    def __hash__(self):
        return hash(self.__u) ^ self.__n

    def __getitem__(self, i):
        """
        EXAMPLES::

            sage: R.<t> = LaurentSeriesRing(QQ)
            sage: f = -5/t^(10) + t + t^2 - 10/3*t^3; f
            -5*t^-10 + t + t^2 - 10/3*t^3
            sage: f[-10]
            -5
            sage: f[1]
            1
            sage: f[3]
            -10/3
            sage: f[-9]
            0
            sage: f = -5/t^(10) + 1/3 + t + t^2 - 10/3*t^3 + O(t^5); f
            -5*t^-10 + 1/3 + t + t^2 - 10/3*t^3 + O(t^5)

        Slicing can be used to truncate, keeping the same precision::

            sage: f[:2]
            -5*t^-10 + 1/3 + t + O(t^5)

        Any other kind of slicing is an error, see :issue:`18940`::

            sage: f[-10:2:2]
            Traceback (most recent call last):
            ...
            IndexError: polynomial slicing with a step is not defined

            sage: f[0:]
            Traceback (most recent call last):
            ...
            IndexError: polynomial slicing with a start is not defined
        """
        if isinstance(i, slice):
            start, stop, step = i.start, i.stop, i.step
            if step is not None:
                raise IndexError("polynomial slicing with a step is not defined")
            if start is not None:
                raise IndexError("polynomial slicing with a start is not defined")
            if stop > self.__u.degree() or stop is None:
                stop = self.__u.degree()
            f = self.__u[:stop - self.__n]
            return type(self)(self._parent, f, self.__n)

        return self.__u[i - self.__n]

    def __iter__(self):
        """
        Iterate through the coefficients from the first nonzero one to the
        last nonzero one.

        EXAMPLES::

            sage: R.<t> = LaurentSeriesRing(QQ)
            sage: f = -5/t^(2) + t + t^2 - 10/3*t^3; f
            -5*t^-2 + t + t^2 - 10/3*t^3
            sage: for a in f: print(a)
            -5
            0
            0
            1
            1
            -10/3
        """
        return iter(self.__u)

    def list(self):
        """
        EXAMPLES::

            sage: R.<t> = LaurentSeriesRing(QQ)
            sage: f = -5/t^(2) + t + t^2 - 10/3*t^3
            sage: f.list()
            [-5, 0, 0, 1, 1, -10/3]
        """
        return self.__u.list()

    def coefficients(self):
        """
        Return the nonzero coefficients of ``self``.

        EXAMPLES::

            sage: R.<t> = LaurentSeriesRing(QQ)
            sage: f = -5/t^(2) + t + t^2 - 10/3*t^3
            sage: f.coefficients()
            [-5, 1, 1, -10/3]
        """
        zero = self._parent.base_ring().zero()
        return [c for c in self.list() if c != zero]

    def residue(self):
        r"""
        Return the residue of ``self``.

        Consider the Laurent series

        .. MATH::

            f = \sum_{n \in \ZZ} a_n t^n
            = \cdots + \frac{a_{-2}}{t^2} + \frac{a_{-1}}{t} + a_0
            + a_1 t + a_2 t^2 + \cdots,

        then the residue of `f` is `a_{-1}`. Alternatively this is the
        coefficient of `1/t`.

        EXAMPLES::

            sage: t = LaurentSeriesRing(ZZ,'t').gen()
            sage: f = 1/t**2 + 2/t + 3 + 4*t
            sage: f.residue()
            2
            sage: f = t + t**2
            sage: f.residue()
            0
            sage: f.residue().parent()
            Integer Ring
        """
        return self[-1]

    def exponents(self):
        """
        Return the exponents appearing in ``self`` with nonzero coefficients.

        EXAMPLES::

            sage: R.<t> = LaurentSeriesRing(QQ)
            sage: f = -5/t^(2) + t + t^2 - 10/3*t^3
            sage: f.exponents()
            [-2, 1, 2, 3]
        """
        zero = self._parent.base_ring().zero()
        v = self.valuation()
        return [i+v for i,val in enumerate(self.list()) if val != zero]

    def laurent_polynomial(self):
        """
        Return the corresponding Laurent polynomial.

        EXAMPLES::

            sage: R.<t> = LaurentSeriesRing(QQ)
            sage: f = t^-3 + t + 7*t^2 + O(t^5)
            sage: g = f.laurent_polynomial(); g
            t^-3 + t + 7*t^2
            sage: g.parent()
            Univariate Laurent Polynomial Ring in t over Rational Field
        """
        R = self._parent.laurent_polynomial_ring()
        return R(self.__u.polynomial()) * R.gen()**(self.__n)

    def lift_to_precision(self, absprec=None):
        """
        Return a congruent Laurent series with absolute precision at least
        ``absprec``.

        INPUT:

        - ``absprec`` -- integer or ``None`` (default: ``None``); the
          absolute precision of the result. If ``None``, lifts to an exact
          element.

        EXAMPLES::

            sage: # needs sage.rings.finite_rings
            sage: A.<t> = LaurentSeriesRing(GF(5))
            sage: x = t^(-1) + t^2 + O(t^5)
            sage: x.lift_to_precision(10)
            t^-1 + t^2 + O(t^10)
            sage: x.lift_to_precision()
            t^-1 + t^2
        """
        if absprec is not None and absprec <= self.precision_absolute():
            return self

        exact = self._parent(0) if self.is_zero() else self._parent(self.list()) << self.__n
        if absprec is None:
            return exact
        else:
            return exact.add_bigoh(absprec)

    def __setitem__(self, n, value):
        """
        EXAMPLES::

            sage: R.<t> = LaurentSeriesRing(QQ)
            sage: f = t^2 + t^3 + O(t^10)
            sage: f[2] = 5
            Traceback (most recent call last):
            ...
            IndexError: Laurent series are immutable
        """
        raise IndexError("Laurent series are immutable")

    def _unsafe_mutate(self, i, value):
        """
        Never use this unless you really know what you are doing.

        .. WARNING::

           This could easily introduce subtle bugs, since Sage assumes
           everywhere that Laurent series are immutable. It's OK to use
           this if you really know what you're doing.

        EXAMPLES::

            sage: R.<t> = LaurentSeriesRing(QQ)
            sage: f = t^2 + t^3 + O(t^10); f
            t^2 + t^3 + O(t^10)
            sage: f._unsafe_mutate(2, -3)
            sage: f
            -3*t^2 + t^3 + O(t^10)
        """
        j = i - self.__n
        if j >= 0:
            self.__u._unsafe_mutate(j, value)
        else: # off to the left
            if value != 0:
                self.__n = self.__n + j
                R = self._parent.base_ring()
                coeffs = [value] + [R(0) for _ in range(1,-j)] + self.__u.list()
                self.__u = self.__u._parent(coeffs)
        self._normalize()

    cpdef _add_(self, right_m):
        """
        Add two power series with the same parent.

        EXAMPLES::

            sage: R.<t> = LaurentSeriesRing(QQ)
            sage: t + t
            2*t
            sage: f = 1/t + t^2 + t^3 - 17/3 * t^4 + O(t^5)
            sage: g = 1/(1-t + O(t^7)); g
            1 + t + t^2 + t^3 + t^4 + t^5 + t^6 + O(t^7)
            sage: f + g
            t^-1 + 1 + t + 2*t^2 + 2*t^3 - 14/3*t^4 + O(t^5)
            sage: f + 0
            t^-1 + t^2 + t^3 - 17/3*t^4 + O(t^5)
            sage: 0 + f
            t^-1 + t^2 + t^3 - 17/3*t^4 + O(t^5)
            sage: R(0) + R(0)
            0
            sage: (t^3 + O(t^10)) + (t^-3 +O(t^9))
            t^-3 + t^3 + O(t^9)

        ALGORITHM: Shift the unit parts to align them, then add.

        TESTS:

        Verify that :issue:`35860` is fixed::

            sage: R.<t> = LaurentPolynomialRing(ZZ)
            sage: sqrt(t^2) + t^-1
            t^-1 + t
        """
        cdef LaurentSeries right = <LaurentSeries>right_m
        cdef long m

        # 1. Special case when one or the other is 0.
        if not right:
            return self.add_bigoh(right.prec())
        if not self:
            return right.add_bigoh(self.prec())

        # 2. Align the unit parts.
        if self.__n < right.__n:
            m = self.__n
            f1 = self.__u
            f2 = right.__u << right.__n - m
        elif self.__n > right.__n:
            m = right.__n
            f1 = self.__u << self.__n - m
            f2 = right.__u
        else:
            m = self.__n
            f1 = self.__u
            f2 = right.__u
        # 3. Add
        return type(self)(self._parent, f1 + f2, m)

    cpdef _sub_(self, right_m):
        """
        Subtract two power series with the same parent.

        EXAMPLES::

            sage: R.<t> = LaurentSeriesRing(QQ)
            sage: t - t
            0
            sage: t^5 + 2 * t^-5
            2*t^-5 + t^5

        ALGORITHM: Shift the unit parts to align them, then subtract.
        """
        cdef LaurentSeries right = <LaurentSeries>right_m
        cdef long m

        # 1. Special case when one or the other is 0.
        if not right:
            return self.add_bigoh(right.prec())
        if not self:
            return -right.add_bigoh(self.prec())

        # 2. Align the unit parts.
        if self.__n < right.__n:
            m = self.__n
            f1 = self.__u
            f2 = right.__u << right.__n - m
        else:
            m = right.__n
            f1 = self.__u << self.__n - m
            f2 = right.__u
        # 3. Subtract
        return type(self)(self._parent, f1 - f2, m)

    def add_bigoh(self, prec):
        """
        Return the truncated series at chosen precision ``prec``.

        See also :meth:`O`.

        INPUT:

        - ``prec`` -- the precision of the series as an integer

        EXAMPLES::

            sage: R.<t> = LaurentSeriesRing(QQ)
            sage: f = t^2 + t^3 + O(t^10); f
            t^2 + t^3 + O(t^10)
            sage: f.add_bigoh(5)
            t^2 + t^3 + O(t^5)

        TESTS:

        Check that :issue:`28239` is fixed::

            sage: (t^(-2)).add_bigoh(-1)
            t^-2 + O(t^-1)
            sage: (t^(-2)).add_bigoh(-2)
            O(t^-2)
            sage: (t^(-2)).add_bigoh(-3)
            O(t^-3)
        """
        if prec is infinity or prec >= self.prec():
            return self
        P = self._parent
        if not self or prec < self.__n:
            return type(self)(P, P._power_series_ring(0, prec=0), prec)
        u = self.__u.add_bigoh(prec - self.__n)
        return type(self)(P, u, self.__n)

    def O(self, prec):
        r"""
        Return the Laurent series of precision at most ``prec`` obtained by
        adding `O(q^\text{prec})`, where `q` is the variable.

        The precision of ``self`` and the integer ``prec`` can be arbitrary. The
        resulting Laurent series will have precision equal to the minimum of
        the precision of ``self`` and ``prec``. The term `O(q^\text{prec})` is the
        zero series with precision ``prec``.

        See also :meth:`add_bigoh`.

        EXAMPLES::

            sage: R.<t> = LaurentSeriesRing(QQ)
            sage: f = t^-5 + t^-4 + t^3 + O(t^10); f
            t^-5 + t^-4 + t^3 + O(t^10)
            sage: f.O(-4)
            t^-5 + O(t^-4)
            sage: f.O(15)
            t^-5 + t^-4 + t^3 + O(t^10)
        """
        return self.add_bigoh(prec)

    def degree(self):
        """
        Return the degree of a polynomial equivalent to this power series
        modulo big oh of the precision.

        EXAMPLES::

            sage: x = Frac(QQ[['x']]).0
            sage: g = x^2 - x^4 + O(x^8)
            sage: g.degree()
            4
            sage: g = -10/x^5 + x^2 - x^4 + O(x^8)
            sage: g.degree()
            4
            sage: (x^-2 + O(x^0)).degree()
            -2
        """
        return self.__u.degree() + self.__n

    def __neg__(self):
        """
        EXAMPLES::

            sage: R.<t> = LaurentSeriesRing(QQ)
            sage: -(1+t^5)
            -1 - t^5
            sage: -(1/(1+t+O(t^5)))
            -1 + t - t^2 + t^3 - t^4 + O(t^5)
        """
        return type(self)(self._parent, -self.__u, self.__n)

    cpdef _mul_(self, right_r):
        """
        EXAMPLES::

            sage: x = Frac(QQ[['x']]).0
            sage: f = 1/x^3 + x + x^2 + 3*x^4 + O(x^7)
            sage: g = 1 - x + x^2 - x^4 + O(x^8)
            sage: f*g
            x^-3 - x^-2 + x^-1 + 4*x^4 + O(x^5)
        """
        cdef LaurentSeries right = <LaurentSeries>right_r
        return type(self)(self._parent,
                          self.__u * right.__u,
                          self.__n + right.__n)

    cpdef _rmul_(self, Element c):
        return type(self)(self._parent, self.__u._rmul_(c), self.__n)

    cpdef _lmul_(self, Element c):
        return type(self)(self._parent, self.__u._lmul_(c), self.__n)

    def __pow__(_self, r, dummy):
        """
        EXAMPLES::

            sage: x = Frac(QQ[['x']]).0
            sage: f = x + x^2 + 3*x^4 + O(x^7)
            sage: g = 1/x^10 - x + x^2 - x^4 + O(x^8)
            sage: f^7
            x^7 + 7*x^8 + 21*x^9 + 56*x^10 + 161*x^11 + 336*x^12 + O(x^13)
            sage: g^7
            x^-70 - 7*x^-59 + 7*x^-58 - 7*x^-56 + O(x^-52)
            sage: g^(1/2)
            x^-5 - 1/2*x^6 + 1/2*x^7 - 1/2*x^9 + O(x^13)
            sage: g^(1/5)
            x^-2 - 1/5*x^9 + 1/5*x^10 - 1/5*x^12 + O(x^16)
            sage: g^(2/5)
            x^-4 - 2/5*x^7 + 2/5*x^8 - 2/5*x^10 + O(x^14)
            sage: h = x^2 + 2*x^4 + x^6
            sage: h^(1/2)
            x + x^3
        """
        cdef LaurentSeries self = _self

        try:
            right = QQ.coerce(r)
        except TypeError:
            raise ValueError("exponent must be a rational number")

        if right.denominator() == 1:
            right = right.numerator()
            return type(self)(self._parent, self.__u**right, self.__n*right)

        if self.is_zero():
            return self._parent(0).O(self.prec()*right)

        d = right.denominator()
        n = right.numerator()

        val = self.valuation()

        if val % d:
            raise ValueError("power series valuation would be fractional")

        u = self.valuation_zero_part().nth_root(d)

        s = type(self)(self._parent, u, val // d)

        return s**n

    def shift(self, k):
        r"""
        Return this Laurent series multiplied by the power `t^n`.
        Does not change this series.

        .. NOTE::

           Despite the fact that higher order terms are printed to the
           right in a power series, right shifting decreases the
           powers of `t`, while left shifting increases
           them. This is to be consistent with polynomials, integers,
           etc.

        EXAMPLES::

            sage: R.<t> = LaurentSeriesRing(QQ['y'])
            sage: f = (t+t^-1)^4; f
            t^-4 + 4*t^-2 + 6 + 4*t^2 + t^4
            sage: f.shift(10)
            t^6 + 4*t^8 + 6*t^10 + 4*t^12 + t^14
            sage: f >> 10
            t^-14 + 4*t^-12 + 6*t^-10 + 4*t^-8 + t^-6
            sage: t << 4
            t^5
            sage: t + O(t^3) >> 4
            t^-3 + O(t^-1)

        AUTHORS:

        - Robert Bradshaw (2007-04-18)
        """
        return type(self)(self._parent, self.__u, self.__n + k)

    def __lshift__(LaurentSeries self, k):
        return type(self)(self._parent, self.__u, self.__n + k)

    def __rshift__(LaurentSeries self, k):
        return type(self)(self._parent, self.__u, self.__n - k)

    def truncate(self, long n):
        r"""
        Return the Laurent series of degree ` < n` which is
        equivalent to ``self`` modulo `x^n`.

        EXAMPLES::

            sage: A.<x> = LaurentSeriesRing(ZZ)
            sage: f = 1/(1-x)
            sage: f
            1 + x + x^2 + x^3 + x^4 + x^5 + x^6 + x^7 + x^8 + x^9 + x^10 + x^11
             + x^12 + x^13 + x^14 + x^15 + x^16 + x^17 + x^18 + x^19 + O(x^20)
            sage: f.truncate(10)
            1 + x + x^2 + x^3 + x^4 + x^5 + x^6 + x^7 + x^8 + x^9
        """
        if n <= self.__n:
            return self._parent.zero()
        else:
            return type(self)(self._parent, self.__u.truncate(n - self.__n), self.__n)

    def truncate_laurentseries(self, long n):
        r"""
        Replace any terms of degree >= n by big oh.

        EXAMPLES::

            sage: A.<x> = LaurentSeriesRing(ZZ)
            sage: f = 1/(1-x)
            sage: f
            1 + x + x^2 + x^3 + x^4 + x^5 + x^6 + x^7 + x^8 + x^9 + x^10 + x^11
             + x^12 + x^13 + x^14 + x^15 + x^16 + x^17 + x^18 + x^19 + O(x^20)
            sage: f.truncate_laurentseries(10)
            1 + x + x^2 + x^3 + x^4 + x^5 + x^6 + x^7 + x^8 + x^9 + O(x^10)
        """
        if n <= self.__n:
            return self._parent.zero()
        else:
            return type(self)(self._parent, self.__u.truncate_powerseries(n - self.__n), self.__n)

    def truncate_neg(self, long n):
        r"""
        Return the Laurent series equivalent to ``self`` except without any
        degree ``n`` terms.

        This is equivalent to::

            ``self - self.truncate(n)``

        EXAMPLES::

            sage: A.<t> = LaurentSeriesRing(ZZ)
            sage: f = 1/(1-t)
            sage: f.truncate_neg(15)
            t^15 + t^16 + t^17 + t^18 + t^19 + O(t^20)
        """
        return type(self)(self._parent, self.__u >> (n - self.__n), n)

    cpdef _div_(self, right_r):
        """
        EXAMPLES::

            sage: x = Frac(QQ[['x']]).0
            sage: f = x + x^2 + 3*x^4 + O(x^7)
            sage: g = 1/x^7 - x + x^2 - x^4 + O(x^8)
            sage: f/x
            1 + x + 3*x^3 + O(x^6)
            sage: f/g
            x^8 + x^9 + 3*x^11 + O(x^14)
        """
        cdef LaurentSeries right = <LaurentSeries>right_r
        if right.__u.is_zero():
            raise ZeroDivisionError
        try:
            return type(self)(self._parent,
                              self.__u / right.__u,
                              self.__n - right.__n)
        except TypeError as msg:
            # todo: this could also make something in the formal fraction field.
            raise ArithmeticError("division not defined")

    def common_prec(self, other):
        r"""
        Return the minimum precision of ``self`` and ``other``.

        EXAMPLES::

            sage: R.<t> = LaurentSeriesRing(QQ)

        ::

            sage: f = t^(-1) + t + t^2 + O(t^3)
            sage: g = t + t^3 + t^4 + O(t^4)
            sage: f.common_prec(g)
            3
            sage: g.common_prec(f)
            3

        ::

            sage: f = t + t^2 + O(t^3)
            sage: g = t^(-3) + t^2
            sage: f.common_prec(g)
            3
            sage: g.common_prec(f)
            3

        ::

            sage: f = t + t^2
            sage: g = t^2
            sage: f.common_prec(g)
            +Infinity

        ::

            sage: f = t^(-3) + O(t^(-2))
            sage: g = t^(-5) + O(t^(-1))
            sage: f.common_prec(g)
            -2

        ::

            sage: f = O(t^2)
            sage: g = O(t^5)
            sage: f.common_prec(g)
            2
        """
        return min(self.prec(), other.prec())

    def common_valuation(self, other):
        """
        Return the minimum valuation of ``self`` and ``other``.

        EXAMPLES::

            sage: R.<t> = LaurentSeriesRing(QQ)

        ::

            sage: f = t^(-1) + t + t^2 + O(t^3)
            sage: g = t + t^3 + t^4 + O(t^4)
            sage: f.common_valuation(g)
            -1
            sage: g.common_valuation(f)
            -1

        ::

            sage: f = t + t^2 + O(t^3)
            sage: g = t^(-3) + t^2
            sage: f.common_valuation(g)
            -3
            sage: g.common_valuation(f)
            -3

        ::

            sage: f = t + t^2
            sage: g = t^2
            sage: f.common_valuation(g)
            1

        ::

            sage: f = t^(-3) + O(t^(-2))
            sage: g = t^(-5) + O(t^(-1))
            sage: f.common_valuation(g)
            -5

        ::

            sage: f = O(t^2)
            sage: g = O(t^5)
            sage: f.common_valuation(g)
            +Infinity
        """
        return min(self.valuation(), other.valuation())

    cpdef _richcmp_(self, right_r, int op):
        r"""
        Comparison of ``self`` and ``right``.

        We say two approximate Laurent series are equal, if they agree for
        all coefficients up to the *minimum* of the precisions of each.
        Comparison is done in dictionary order from lowest degree to
        highest degree coefficients. This is different than polynomials,
        but consistent with the idea that the variable of a Laurent
        series is considered to be "very small".

        See :meth:`power_series_ring_element._richcmp_` for more
        information.

        EXAMPLES::

            sage: R.<x> = LaurentSeriesRing(QQ)
            sage: f = x^(-1) + 1 + x + O(x^2)
            sage: g = x^(-1) + 1 + O(x)
            sage: f == g
            True

        ::

            sage: f = x^(-1) + 1 + x + O(x^2)
            sage: g = x^(-1) + 2 + O(x)
            sage: f == g
            False
            sage: f < g
            True
            sage: f > g
            False

        ::

            sage: f = x^(-2) + 1 + x + O(x^2)
            sage: g = x^(-1) + 2 + O(x)
            sage: f == g
            False
            sage: f < g
            False
            sage: f > g
            True

        Check that :issue:`19664` is fixed::

            sage: R.<x> = LaurentSeriesRing(RR)
            sage: x^(10^9) > 0
            True
        """
        cdef LaurentSeries right = <LaurentSeries>right_r

        val = self.common_valuation(right)
        if val is infinity:
            return rich_to_bool(op, 0)  # Both arguments are zero

        cdef long deg = max(self.degree(), right.degree())
        prec = self.common_prec(right)
        if deg >= prec:
            deg = prec - 1

        cdef long i
        for i in range(val, deg + 1):
            li = self[i]
            ri = right[i]
            if li != ri:
                return richcmp_not_equal(li, ri, op)
        return rich_to_bool(op, 0)

    def valuation_zero_part(self):
        """
        EXAMPLES::

            sage: x = Frac(QQ[['x']]).0
            sage: f = x + x^2 + 3*x^4 + O(x^7)
            sage: f/x
            1 + x + 3*x^3 + O(x^6)
            sage: f.valuation_zero_part()
            1 + x + 3*x^3 + O(x^6)
            sage: g = 1/x^7 - x + x^2 - x^4 + O(x^8)
            sage: g.valuation_zero_part()
            1 - x^8 + x^9 - x^11 + O(x^15)
        """
        return self.__u

    def valuation(self):
        """
        EXAMPLES::

            sage: R.<x> = LaurentSeriesRing(QQ)
            sage: f = 1/x + x^2 + 3*x^4 + O(x^7)
            sage: g = 1 - x + x^2 - x^4 + O(x^8)
            sage: f.valuation()
            -1
            sage: g.valuation()
            0

        Note that the valuation of an element undistinguishable from
        zero is infinite::

            sage: h = f - f; h
            O(x^7)
            sage: h.valuation()
            +Infinity

        TESTS:

        The valuation of the zero element is ``+Infinity``
        (see :issue:`15088`)::

            sage: zero = R(0)
            sage: zero.valuation()
            +Infinity
        """
        if self.is_zero():
            return infinity
        return self.__n

    def variable(self):
        """
        EXAMPLES::

            sage: x = Frac(QQ[['x']]).0
            sage: f = 1/x + x^2 + 3*x^4 + O(x^7)
            sage: f.variable()
            'x'
        """
        return self._parent.variable_name()

    def prec(self):
        """
        This function returns the n so that the Laurent series is of the
        form (stuff) + `O(t^n)`. It doesn't matter how many
        negative powers appear in the expansion. In particular, prec could
        be negative.

        EXAMPLES::

            sage: x = Frac(QQ[['x']]).0
            sage: f = x^2 + 3*x^4 + O(x^7)
            sage: f.prec()
            7
            sage: g = 1/x^10 - x + x^2 - x^4 + O(x^8)
            sage: g.prec()
            8
        """
        return self.__u.prec() + self.__n

    def precision_absolute(self):
        """
        Return the absolute precision of this series.

        By definition, the absolute precision of
        `...+O(x^r)` is `r`.

        EXAMPLES::

            sage: R.<t> = ZZ[[]]
            sage: (t^2 + O(t^3)).precision_absolute()
            3
            sage: (1 - t^2 + O(t^100)).precision_absolute()
            100
        """
        return self.prec()

    def precision_relative(self):
        """
        Return the relative precision of this series, that
        is the difference between its absolute precision
        and its valuation.

        By convention, the relative precision of `0` (or
        `O(x^r)` for any `r`) is `0`.

        EXAMPLES::

            sage: R.<t> = ZZ[[]]
            sage: (t^2 + O(t^3)).precision_relative()
            1
            sage: (1 - t^2 + O(t^100)).precision_relative()
            100
            sage: O(t^4).precision_relative()
            0
        """
        if self.is_zero():
            return 0
        else:
            return self.prec() - self.valuation()

    def __copy__(self):
        return type(self)(self._parent, self.__u.__copy__(), self.__n)

    def reverse(self, precision=None):
        """
        Return the reverse of f, i.e., the series g such that g(f(x)) = x.
        Given an optional argument ``precision``, return the reverse with given
        precision (note that the reverse can have precision at most
        ``f.prec()``).  If ``f`` has infinite precision, and the argument
        ``precision`` is not given, then the precision of the reverse defaults
        to the default precision of ``f.parent()``.

        Note that this is only possible if the valuation of ``self`` is exactly
        1.

        The implementation depends on the underlying power series element
        implementing a reverse method.

        EXAMPLES::

            sage: # needs sage.libs.pari
            sage: R.<x> = Frac(QQ[['x']])
            sage: f = 2*x + 3*x^2 - x^4 + O(x^5)
            sage: g = f.reverse()
            sage: g
            1/2*x - 3/8*x^2 + 9/16*x^3 - 131/128*x^4 + O(x^5)
            sage: f(g)
            x + O(x^5)
            sage: g(f)
            x + O(x^5)

            sage: # needs sage.libs.pari
            sage: A.<t> = LaurentSeriesRing(ZZ)
            sage: a = t - t^2 - 2*t^4 + t^5 + O(t^6)
            sage: b = a.reverse(); b
            t + t^2 + 2*t^3 + 7*t^4 + 25*t^5 + O(t^6)
            sage: a(b)
            t + O(t^6)
            sage: b(a)
            t + O(t^6)

            sage: # needs sage.libs.pari
            sage: B.<b,c> = ZZ[ ]
            sage: A.<t> = LaurentSeriesRing(B)
            sage: f = t + b*t^2 + c*t^3 + O(t^4)
            sage: g = f.reverse(); g
            t - b*t^2 + (2*b^2 - c)*t^3 + O(t^4)
            sage: f(g)
            t + O(t^4)
            sage: g(f)
            t + O(t^4)

            sage: # needs sage.libs.pari
            sage: A.<t> = PowerSeriesRing(ZZ)
            sage: B.<s> = LaurentSeriesRing(A)
            sage: f = (1 - 3*t + 4*t^3 + O(t^4))*s + (2 + t + t^2 + O(t^3))*s^2 + O(s^3)
            sage: set_verbose(1)
            sage: g = f.reverse(); g
            verbose 1 (<module>) passing to pari failed; trying Lagrange inversion
            (1 + 3*t + 9*t^2 + 23*t^3 + O(t^4))*s + (-2 - 19*t - 118*t^2 + O(t^3))*s^2 + O(s^3)
            sage: set_verbose(0)
            sage: f(g) == g(f) == s
            True

        If the leading coefficient is not a unit, we pass to its fraction
        field if possible::

            sage: # needs sage.libs.pari
            sage: A.<t> = LaurentSeriesRing(ZZ)
            sage: a = 2*t - 4*t^2 + t^4 - t^5 + O(t^6)
            sage: a.reverse()
            1/2*t + 1/2*t^2 + t^3 + 79/32*t^4 + 437/64*t^5 + O(t^6)

            sage: # needs sage.libs.pari
            sage: B.<b> = PolynomialRing(ZZ)
            sage: A.<t> = LaurentSeriesRing(B)
            sage: f = 2*b*t + b*t^2 + 3*b^2*t^3 + O(t^4)
            sage: g = f.reverse(); g
            1/(2*b)*t - 1/(8*b^2)*t^2 + ((-3*b + 1)/(16*b^3))*t^3 + O(t^4)
            sage: f(g)
            t + O(t^4)
            sage: g(f)
            t + O(t^4)

        We can handle some base rings of positive characteristic::

            sage: # needs sage.libs.pari
            sage: A8.<t> = LaurentSeriesRing(Zmod(8))
            sage: a = t - 15*t^2 - 2*t^4 + t^5 + O(t^6)
            sage: b = a.reverse(); b
            t + 7*t^2 + 2*t^3 + 5*t^4 + t^5 + O(t^6)
            sage: a(b)
            t + O(t^6)
            sage: b(a)
            t + O(t^6)

        The optional argument ``precision`` sets the precision of the output::

            sage: # needs sage.libs.pari
            sage: R.<x> = LaurentSeriesRing(QQ)
            sage: f = 2*x + 3*x^2 - 7*x^3 + x^4 + O(x^5)
            sage: g = f.reverse(precision=3); g
            1/2*x - 3/8*x^2 + O(x^3)
            sage: f(g)
            x + O(x^3)
            sage: g(f)
            x + O(x^3)

        If the input series has infinite precision, the precision of the
        output is automatically set to the default precision of the parent
        ring::

            sage: # needs sage.libs.pari
            sage: R.<x> = LaurentSeriesRing(QQ, default_prec=20)
            sage: (x - x^2).reverse()  # get some Catalan numbers
            x + x^2 + 2*x^3 + 5*x^4 + 14*x^5 + 42*x^6 + 132*x^7 + 429*x^8 + 1430*x^9
             + 4862*x^10 + 16796*x^11 + 58786*x^12 + 208012*x^13 + 742900*x^14
             + 2674440*x^15 + 9694845*x^16 + 35357670*x^17 + 129644790*x^18
             + 477638700*x^19 + O(x^20)
            sage: (x - x^2).reverse(precision=3)
            x + x^2 + O(x^3)

        TESTS::

            sage: R.<x> = LaurentSeriesRing(QQ)
            sage: f = 1 + 2*x + 3*x^2 - x^4 + O(x^5)
            sage: f.reverse()
            Traceback (most recent call last):
            ...
            ValueError: Series must have valuation one for reversion.
        """
        val = self.valuation()
        if val != 1:
            raise ValueError("Series must have valuation one for reversion.")
        u = self.valuation_zero_part()
        u = u.parent().gen(0) * u

        rev = u.reverse(precision=precision)

        if rev.parent() == u.parent():
            return self._parent(rev)
        else:
            P = self._parent.change_ring(rev.parent().base_ring())
            return P(rev)

    def derivative(self, *args):
        """
        The formal derivative of this Laurent series, with respect to
        variables supplied in args.

        Multiple variables and iteration counts may be supplied; see
        documentation for the global derivative() function for more
        details.

        .. SEEALSO::

           :meth:`_derivative`

        EXAMPLES::

            sage: R.<x> = LaurentSeriesRing(QQ)
            sage: g = 1/x^10 - x + x^2 - x^4 + O(x^8)
            sage: g.derivative()
            -10*x^-11 - 1 + 2*x - 4*x^3 + O(x^7)
            sage: g.derivative(x)
            -10*x^-11 - 1 + 2*x - 4*x^3 + O(x^7)

        ::

            sage: R.<t> = PolynomialRing(ZZ)
            sage: S.<x> = LaurentSeriesRing(R)
            sage: f = 2*t/x + (3*t^2 + 6*t)*x + O(x^2)
            sage: f.derivative()
            -2*t*x^-2 + (3*t^2 + 6*t) + O(x)
            sage: f.derivative(x)
            -2*t*x^-2 + (3*t^2 + 6*t) + O(x)
            sage: f.derivative(t)
            2*x^-1 + (6*t + 6)*x + O(x^2)
        """
        return multi_derivative(self, args)

    def _derivative(self, var=None):
        """
        The formal derivative of this Laurent series with respect to var.

        If var is None or the generator of this ring, it's the formal
        derivative as expected. Otherwise, _derivative(var) gets called
        recursively on each coefficient.

        .. SEEALSO::

           :meth:`derivative`

        EXAMPLES::

            sage: x = Frac(QQ[['x']]).0
            sage: f = x^2 + 3*x^4 + O(x^7)
            sage: f._derivative()
            2*x + 12*x^3 + O(x^6)
            sage: f._derivative(x)
            2*x + 12*x^3 + O(x^6)
            sage: g = 1/x^10 - x + x^2 - x^4 + O(x^8)
            sage: g._derivative()
            -10*x^-11 - 1 + 2*x - 4*x^3 + O(x^7)

        Differentiating with respect to something other than the generator
        gets recursed into the base ring::

            sage: R.<t> = PolynomialRing(ZZ)
            sage: S.<x> = LaurentSeriesRing(R)
            sage: f = 2*t/x + (3*t^2 + 6*t)*x + O(x^2)
            sage: f._derivative(t)
            2*x^-1 + (6*t + 6)*x + O(x^2)

        TESTS::

            sage: y = var('y')                                                          # needs sage.symbolic
            sage: f.derivative(y)                                                       # needs sage.symbolic
            Traceback (most recent call last):
            ...
            ValueError: cannot differentiate with respect to y
        """
        if var is not None and var != self._parent.gen():
            try:
                # call _derivative() recursively on coefficients
                u = [coeff._derivative(var) for coeff in self.__u.list()]
                u = self._parent._power_series_ring(u, self.__u.prec())
                return type(self)(self._parent, u, self.__n)
            except AttributeError:
                raise ValueError("cannot differentiate with respect to {}".format(var))

        # compute formal derivative with respect to generator
        if self.is_zero():
            return type(self)(self._parent, 0, self.__u.prec() - 1)
        cdef long m, n = self.__n
        a = self.__u.list()
        v = [(n+m)*a[m] for m from 0 <= m < len(a)]
        u = self._parent._power_series_ring(v, self.__u.prec())
        return type(self)(self._parent, u, n-1)

    def integral(self):
        r"""
        The formal integral of this Laurent series with 0 constant term.

        EXAMPLES: The integral may or may not be defined if the base ring
        is not a field.

        ::

            sage: t = LaurentSeriesRing(ZZ, 't').0
            sage: f = 2*t^-3 + 3*t^2 + O(t^4)
            sage: f.integral()
            -t^-2 + t^3 + O(t^5)

        ::

            sage: f = t^3
            sage: f.integral()
            Traceback (most recent call last):
            ...
            ArithmeticError: Coefficients of integral cannot be coerced into the base ring

        The integral of 1/t is `\log(t)`, which is not given by a
        Laurent series::

            sage: t = Frac(QQ[['t']]).0
            sage: f = -1/t^3 - 31/t + O(t^3)
            sage: f.integral()
            Traceback (most recent call last):
            ...
            ArithmeticError: The integral of is not a Laurent series, since t^-1 has nonzero coefficient.

        Another example with just one negative coefficient::

            sage: A.<t> = QQ[[]]
            sage: f = -2*t^(-4) + O(t^8)
            sage: f.integral()
            2/3*t^-3 + O(t^9)
            sage: f.integral().derivative() == f
            True
        """
        cdef long i, n = self.__n
        a = self.__u.list()
        if self[-1] != 0:
            raise ArithmeticError("The integral of is not a Laurent series, since t^-1 has nonzero coefficient.")

        if n < 0:
            v = [a[i]/(n+i+1) for i in range(min(-1-n,len(a)))] + [0]
        else:
            v = []
        v += [a[i]/(n+i+1) for i in range(max(-n,0), len(a))]
        try:
            u = self._parent._power_series_ring(v, self.__u.prec())
        except TypeError:
            raise ArithmeticError("Coefficients of integral cannot be coerced into the base ring")
        return type(self)(self._parent, u, n+1)

    def nth_root(self, long n, prec=None):
        r"""
        Return the ``n``-th root of this Laurent power series.

        INPUT:

        - ``n`` -- integer

        - ``prec`` -- integer (optional); precision of the result. Though, if
          this series has finite precision, then the result cannot have larger
          precision.

        EXAMPLES::

            sage: R.<x> = LaurentSeriesRing(QQ)
            sage: (x^-2 + 1 + x).nth_root(2)
            x^-1 + 1/2*x + 1/2*x^2 - ... - 19437/65536*x^18 + O(x^19)
            sage: (x^-2 + 1 + x).nth_root(2)**2
            x^-2 + 1 + x + O(x^18)

            sage: # needs sage.modular
            sage: j = j_invariant_qexp()
            sage: q = j.parent().gen()
            sage: j(q^3).nth_root(3)
            q^-1 + 248*q^2 + 4124*q^5 + ... + O(q^29)
            sage: (j(q^2) - 1728).nth_root(2)
            q^-1 - 492*q - 22590*q^3 - ... + O(q^19)
        """
        if prec is None:
            prec = self.prec()
            if prec is infinity:
                prec = self.parent().default_prec()
        else:
            prec = min(self.prec(), prec)

        if n <= 0:
            raise ValueError('n must be positive')

        i = self.valuation()
        if i % n:
            raise ValueError('valuation must be divisible by n')

        q = self.__u.nth_root(n, prec)
        return type(self)(self._parent, q + self.parent()(0).O(prec), i // n)

    def power_series(self):
        """
        Convert this Laurent series to a power series.

        An error is raised if the Laurent series has a term (or an error
        term `O(x^k)`) whose exponent is negative.

        EXAMPLES::

            sage: R.<t> = LaurentSeriesRing(ZZ)
            sage: f = 1/(1-t+O(t^10)); f.parent()
            Laurent Series Ring in t over Integer Ring
            sage: g = f.power_series(); g
            1 + t + t^2 + t^3 + t^4 + t^5 + t^6 + t^7 + t^8 + t^9 + O(t^10)
            sage: parent(g)
            Power Series Ring in t over Integer Ring
            sage: f = 3/t^2 +  t^2 + t^3 + O(t^10)
            sage: f.power_series()
            Traceback (most recent call last):
            ...
            TypeError: self is not a power series

        TESTS:

        Check whether a polynomial over a Laurent series ring is contained in the
        polynomial ring over the power series ring (see :issue:`19459`):

            sage: # needs sage.rings.finite_rings
            sage: L.<t> = LaurentSeriesRing(GF(2))
            sage: R.<x,y> = PolynomialRing(L)
            sage: S.<x,y> = PolynomialRing(L._power_series_ring)
            sage: t**(-1)*x*y in S
            False

        There used to be an issue with non-canonical representations of zero,
        see :issue:`31383`::

            sage: S.<x> = PowerSeriesRing(QQ)
            sage: L = Frac(S)
            sage: s = L(O(x^2))
            sage: (s*x^(-1)).power_series()
            O(x^1)
            sage: (s*x^(-2)).power_series()
            O(x^0)
            sage: (s*x^(-3)).power_series()
            Traceback (most recent call last):
            ...
            TypeError: self is not a power series

        Test for :issue:`32440`::

            sage: L.<x> = LaurentSeriesRing(QQ, implementation='pari')                  # needs sage.libs.pari
            sage: (x + O(x^3)).power_series()                                           # needs sage.libs.pari
            x + O(x^3)
        """
        if self.__n < 0:
            if self.__u.is_zero() and self.__u.prec() >= - self.__n:
                return self.__u >> (- self.__n)
            else:
                raise TypeError("self is not a power series")
        return self.__u << self.__n

    def inverse(self):
        """
        Return the inverse of self, i.e., self^(-1).

        EXAMPLES::

            sage: R.<t> = LaurentSeriesRing(ZZ)
            sage: t.inverse()
            t^-1
            sage: (1-t).inverse()
            1 + t + t^2 + t^3 + t^4 + t^5 + t^6 + t^7 + t^8 + ...
        """
        return ~self

    def __call__(self, *x, **kwds):
        """
        Compute value of this Laurent series at x.

        EXAMPLES::

            sage: P.<x, y> = ZZ[]
            sage: R.<t> = LaurentSeriesRing(P)
            sage: f = x*t^-2 + y*t^2 + O(t^8)
            sage: f(t^3)
            x*t^-6 + y*t^6 + O(t^24)
            sage: f(t=t^3)
            x*t^-6 + y*t^6 + O(t^24)
            sage: f(t + O(t^5))
            x*t^-2 + O(t^2)
            sage: f(y=x)
            x*t^-2 + x*t^2 + O(t^8)
            sage: f(t^3, x=2, y=x + x^2)
            2*t^-6 + (x^2 + x)*t^6 + O(t^24)
            sage: f(t^3, 2, x+x^2)
            2*t^-6 + (x^2 + x)*t^6 + O(t^24)
            sage: f(x=2, t=t^3, y=x + x^2)
            2*t^-6 + (x^2 + x)*t^6 + O(t^24)
            sage: f(2, x + x^2, t=t^3)
            Traceback (most recent call last):
            ...
            ValueError: must not specify t keyword and positional argument

        It is only possible to substitute elements of positive valuation::

            sage: f(t^-2)
            Traceback (most recent call last):
            ...
            ValueError: Can only substitute elements of positive valuation

        Test for :issue:`23928`::

            sage: R.<x> = LaurentSeriesRing(QQ, implementation='pari')                  # needs sage.libs.pari
            sage: f = x.add_bigoh(7)                                                    # needs sage.libs.pari
            sage: f(x)                                                                  # needs sage.libs.pari
            x + O(x^7)
            """
        if len(kwds) >= 1:
            name = self.parent().variable_name()
            if name in kwds: # a keyword specifies the Laurent series generator
                if x:
                    raise ValueError("must not specify %s keyword and positional argument" % name)
                a = self(kwds[name])
                del kwds[name]
                try:
                    return a(**kwds)
                except TypeError:
                    return a
            elif x:       # both keywords and positional arguments
                a = self(*x)
                try:
                    return a(**kwds)
                except TypeError:
                    return a
            else:                  # keywords but no positional arguments
                return self.__u(**kwds)*(self.parent().gen()**self.__n)

        if len(x) == 0:
            return self

        if isinstance(x[0], tuple):
            x = x[0]

        return self.__u(*x)*(x[0]**self.__n)

    def __pari__(self):
        """
        Convert ``self`` to a PARI object.

        TESTS::

            sage: L.<x> = LaurentSeriesRing(QQ)
            sage: f = x + 1/x + O(x^2); f
            x^-1 + x + O(x^2)
            sage: f.__pari__()                                                          # needs sage.libs.pari
            x^-1 + x + O(x^2)

        Check that :issue:`32437` is fixed::

            sage: # needs sage.rings.finite_rings
            sage: F.<u> = GF(257^2)
            sage: R.<t> = LaurentSeriesRing(F)
            sage: g = t + O(t^99)
            sage: f = u*t + O(t^99)
            sage: g(f)  # indirect doctest                                              # needs sage.libs.pari
            u*t + O(t^99)
        """
        f = self.__u
        x = f.parent().gen()
        return f.__pari__() * x.__pari__()**self.__n



# sage_setup: distribution = sagemath-categories
r"""
Finite dimensional modules with basis
"""
# ****************************************************************************
#  Copyright (C) 2008 Teresa Gomez-Diaz (CNRS) <Teresa.Gomez-Diaz@univ-mlv.fr>
#                2011 Nicolas M. Thiery <nthiery at users.sf.net>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#                  http://www.gnu.org/licenses/
# *****************************************************************************

import operator
from sage.categories.category_with_axiom import CategoryWithAxiom, CategoryWithAxiom_over_base_ring
from sage.categories.fields import Fields
from sage.categories.homsets import HomsetsCategory
from sage.categories.tensor import TensorProductsCategory
from sage.misc.cachefunc import cached_method
from sage.misc.lazy_attribute import lazy_attribute


class FiniteDimensionalModulesWithBasis(CategoryWithAxiom_over_base_ring):
    """
    The category of finite dimensional modules with a distinguished basis.

    EXAMPLES::

      sage: C = FiniteDimensionalModulesWithBasis(ZZ); C
      Category of finite dimensional modules with basis over Integer Ring
      sage: sorted(C.super_categories(), key=str)
      [Category of finite dimensional modules over Integer Ring,
       Category of modules with basis over Integer Ring]
      sage: C is Modules(ZZ).WithBasis().FiniteDimensional()
      True

    TESTS::

        sage: TestSuite(C).run()
    """

    class ParentMethods:

        def gens(self) -> tuple:
            """
            Return the generators of ``self``.

            OUTPUT: a tuple containing the basis elements of ``self``

            EXAMPLES::

                sage: F = CombinatorialFreeModule(ZZ, ['a', 'b', 'c'])                  # needs sage.modules
                sage: F.gens()                                                          # needs sage.modules
                (B['a'], B['b'], B['c'])
            """
            return tuple(self.basis())

        def annihilator(self, S, action=operator.mul, side='right', category=None):
            r"""
            Return the annihilator of a finite set.

            INPUT:

            - ``S`` -- a finite set

            - ``action`` -- a function (default: :obj:`operator.mul`)

            - ``side`` -- ``'left'`` or ``'right'`` (default: ``'right'``)

            - ``category`` -- a category

            Assumptions:

            - ``action`` takes elements of ``self`` as first argument
              and elements of ``S`` as second argument;

            - The codomain is any vector space, and ``action`` is
              linear on its first argument; typically it is bilinear;

            - If ``side`` is 'left', this is reversed.

            OUTPUT:

            The subspace of the elements `x` of ``self`` such that
            ``action(x,s) = 0`` for all `s\in S`. If ``side`` is
            'left' replace the above equation by ``action(s,x) = 0``.

            If ``self`` is a ring, ``action`` an action of ``self`` on
            a module `M` and `S` is a subset of `M`, we recover the
            :wikipedia:`Annihilator_%28ring_theory%29`. Similarly this
            can be used to compute torsion or orthogonals.

            .. SEEALSO:: :meth:`annihilator_basis` for lots of examples.

            EXAMPLES::

                sage: # needs sage.modules
                sage: F = FiniteDimensionalAlgebrasWithBasis(QQ).example(); F
                An example of a finite dimensional algebra with basis:
                the path algebra of the Kronecker quiver
                (containing the arrows a:x->y and b:x->y) over Rational Field
                sage: x, y, a, b = F.basis()
                sage: A = F.annihilator([a + 3*b + 2*y]); A
                Free module generated by {0} over Rational Field
                sage: [b.lift() for b in A.basis()]
                [-1/2*a - 3/2*b + x]

            The category can be used to specify other properties of
            this subspace, like that this is a subalgebra::

                sage: # needs sage.modules
                sage: center = F.annihilator(F.basis(), F.bracket,
                ....:                        category=Algebras(QQ).Subobjects())
                sage: (e,) = center.basis()
                sage: e.lift()
                x + y
                sage: e * e == e
                True

            Taking annihilator is order reversing for inclusion::

                sage: # needs sage.modules
                sage: A   = F.annihilator([]);    A  .rename('A')
                sage: Ax  = F.annihilator([x]);   Ax .rename('Ax')
                sage: Ay  = F.annihilator([y]);   Ay .rename('Ay')
                sage: Axy = F.annihilator([x,y]); Axy.rename('Axy')
                sage: P = Poset(([A, Ax, Ay, Axy], attrcall("is_submodule")))           # needs sage.graphs
                sage: sorted(P.cover_relations(), key=str)                              # needs sage.graphs
                [[Ax, A], [Axy, Ax], [Axy, Ay], [Ay, A]]
            """
            return self.submodule(self.annihilator_basis(S, action, side),
                                  already_echelonized=True,
                                  category=category)

        def annihilator_basis(self, S, action=operator.mul, side='right'):
            """
            Return a basis of the annihilator of a finite set of elements.

            INPUT:

            - ``S`` -- a finite set of objects

            - ``action`` -- a function (default: :obj:`operator.mul`)

            - ``side`` -- ``'left'`` or ``'right'`` (default: ``'right'``):
              on which side of ``self`` the elements of `S` acts

            See :meth:`annihilator` for the assumptions and definition
            of the annihilator.

            EXAMPLES:

            By default, the action is the standard `*` operation. So
            our first example is about an algebra::

                sage: # needs sage.graphs sage.modules
                sage: F = FiniteDimensionalAlgebrasWithBasis(QQ).example(); F
                An example of a finite dimensional algebra with basis:
                the path algebra of the Kronecker quiver
                (containing the arrows a:x->y and b:x->y) over Rational Field
                sage: x,y,a,b = F.basis()

            In this algebra, multiplication on the right by `x`
            annihilates all basis elements but `x`::

                sage: x*x, y*x, a*x, b*x                                                # needs sage.graphs sage.modules
                (x, 0, 0, 0)

            So the annihilator is the subspace spanned by `y`, `a`, and `b`::

                sage: F.annihilator_basis([x])                                          # needs sage.graphs sage.modules
                (y, a, b)

            The same holds for `a` and `b`::

                sage: x*a, y*a, a*a, b*a                                                # needs sage.graphs sage.modules
                (a, 0, 0, 0)
                sage: F.annihilator_basis([a])                                          # needs sage.graphs sage.modules
                (y, a, b)

            On the other hand, `y` annihilates only `x`::

                sage: F.annihilator_basis([y])                                          # needs sage.graphs sage.modules
                (x,)

            Here is a non trivial annihilator::

                sage: F.annihilator_basis([a + 3*b + 2*y])                              # needs sage.graphs sage.modules
                (-1/2*a - 3/2*b + x,)

            Let's check it::

                sage: (-1/2*a - 3/2*b + x) * (a + 3*b + 2*y)                            # needs sage.graphs sage.modules
                0

            Doing the same calculations on the left exchanges the
            roles of `x` and `y`::

                sage: # needs sage.graphs sage.modules
                sage: F.annihilator_basis([y], side='left')
                (x, a, b)
                sage: F.annihilator_basis([a], side='left')
                (x, a, b)
                sage: F.annihilator_basis([b], side='left')
                (x, a, b)
                sage: F.annihilator_basis([x], side='left')
                (y,)
                sage: F.annihilator_basis([a + 3*b + 2*x], side='left')
                (-1/2*a - 3/2*b + y,)

            By specifying an inner product, this method can be used to
            compute the orthogonal of a subspace::

                sage: # needs sage.graphs sage.modules
                sage: x,y,a,b = F.basis()
                sage: def scalar(u, v):
                ....:     return vector([sum(u[i]*v[i] for i in F.basis().keys())])
                sage: F.annihilator_basis([x + y, a + b], scalar)
                (x - y, a - b)

            By specifying the standard Lie bracket as action, one can
            compute the commutator of a subspace of `F`::

                sage: F.annihilator_basis([a + b], action=F.bracket)                    # needs sage.graphs sage.modules
                (x + y, a, b)

            In particular one can compute a basis of the center of the
            algebra. In our example, it is reduced to the identity::

                sage: F.annihilator_basis(F.algebra_generators(), action=F.bracket)     # needs sage.graphs sage.modules
                (x + y,)

            But see also
            :meth:`FiniteDimensionalAlgebrasWithBasis.ParentMethods.center_basis`.
            """
            # TODO: optimize this!
            from sage.matrix.constructor import matrix
            if side == 'right':
                action_left = action
                action = lambda b,s: action_left(s, b)

            mat = matrix(self.base_ring(), self.dimension(), 0)
            for s in S:
                mat = mat.augment(matrix(self.base_ring(),
                                         [action(s, b)._vector_() for b in self.basis()]))
            return tuple(map(self.from_vector, mat.left_kernel().basis()))

        @cached_method
        def _dense_free_module(self, base_ring=None):
            """
            Return a dense free module of the same dimension as ``self``.

            INPUT:

            - ``base_ring`` -- a ring or ``None``

            If ``base_ring`` is ``None``, then the base ring of ``self``
            is used.

            This method is mostly used by ``_vector_``

            EXAMPLES::

                sage: C = CombinatorialFreeModule(QQ['x'], ['a','b','c']); C            # needs sage.modules
                Free module generated by {'a', 'b', 'c'} over
                 Univariate Polynomial Ring in x over Rational Field
                sage: C._dense_free_module()                                            # needs sage.modules
                Ambient free module of rank 3 over the principal ideal domain
                 Univariate Polynomial Ring in x over Rational Field
                sage: C._dense_free_module(QQ['x,y'])                                   # needs sage.modules
                Ambient free module of rank 3 over the integral domain
                 Multivariate Polynomial Ring in x, y over Rational Field
            """
            if base_ring is None:
                base_ring = self.base_ring()
            from sage.modules.free_module import FreeModule
            return FreeModule(base_ring, self.dimension())

        def from_vector(self, vector, order=None, coerce=True):
            """
            Build an element of ``self`` from a vector.

            EXAMPLES::

                sage: # needs sage.modules
                sage: p_mult = matrix([[0,0,0], [0,0,-1], [0,0,0]])
                sage: q_mult = matrix([[0,0,1], [0,0,0], [0,0,0]])
                sage: A = algebras.FiniteDimensional(
                ....:         QQ, [p_mult, q_mult, matrix(QQ, 3, 3)], 'p,q,z')
                sage: A.from_vector(vector([1,0,2]))
                p + 2*z
            """
            if order is None:
                try:
                    order = sorted(self.basis().keys())
                except AttributeError: # Not a family, assume it is list-like
                    order = range(self.dimension())
            if not coerce or vector.base_ring() is self.base_ring():
                return self._from_dict({order[i]: c for i,c in vector.items()},
                                       coerce=False)
            R = self.base_ring()
            return self._from_dict({order[i]: R(c) for i,c in vector.items() if R(c)},
                                   coerce=False, remove_zeros=False)

        def echelon_form(self, elements, row_reduced=False, order=None):
            r"""
            Return a basis in echelon form of the subspace spanned by
            a finite set of elements.

            INPUT:

            - ``elements`` -- list or finite iterable of elements of ``self``
            - ``row_reduced`` -- boolean (default: ``False``); whether to
              compute the basis for the row reduced echelon form
            - ``order`` -- (optional) either something that can
              be converted into a tuple or a key function

            OUTPUT:

            A list of elements of ``self`` whose expressions as vectors
            form a matrix in echelon form. If ``base_ring`` is specified,
            then the calculation is achieved in this base ring.

            EXAMPLES::

                sage: # needs sage.modules
                sage: X = CombinatorialFreeModule(QQ, range(3), prefix='x')
                sage: x = X.basis()
                sage: V = X.echelon_form([x[0]-x[1], x[0]-x[2], x[1]-x[2]]); V
                [x[0] - x[2], x[1] - x[2]]
                sage: matrix(list(map(vector, V)))
                [ 1  0 -1]
                [ 0  1 -1]

            ::

                sage: # needs sage.modules
                sage: F = CombinatorialFreeModule(ZZ, [1,2,3,4])
                sage: B = F.basis()
                sage: elements = [B[1]-17*B[2]+6*B[3], B[1]-17*B[2]+B[4]]
                sage: F.echelon_form(elements)
                [B[1] - 17*B[2] + B[4], 6*B[3] - B[4]]

            ::

                sage: F = CombinatorialFreeModule(QQ, ['a','b','c'])                    # needs sage.modules
                sage: a,b,c = F.basis()                                                 # needs sage.modules
                sage: F.echelon_form([8*a+b+10*c, -3*a+b-c, a-b-c])                     # needs sage.modules
                [B['a'] + B['c'], B['b'] + 2*B['c']]

            ::

                sage: R.<x,y> = QQ[]
                sage: C = CombinatorialFreeModule(R, range(3), prefix='x')              # needs sage.modules
                sage: x = C.basis()                                                     # needs sage.modules
                sage: C.echelon_form([x[0] - x[1], 2*x[1] - 2*x[2], x[0] - x[2]])       # needs sage.modules sage.rings.function_field
                [x[0] - x[2], x[1] - x[2]]

            ::

                sage: M = MatrixSpace(QQ, 3, 3)                                         # needs sage.modules
                sage: A = M([[0, 0, 2], [0, 0, 0], [0, 0, 0]])                          # needs sage.modules
                sage: M.echelon_form([A, A])                                            # needs sage.modules
                [
                [0 0 1]
                [0 0 0]
                [0 0 0]
                ]

            TESTS:

            We convert the input elements to ``self``::

                sage: E.<x,y,z> = ExteriorAlgebra(QQ)                                   # needs sage.modules
                sage: E.echelon_form([1, x + 2])                                        # needs sage.modules
                [1, x]
            """
            # Make sure elements consists of elements of ``self``
            elements = [self(y) for y in elements]
            if order is not None:
                order = self._compute_support_order(elements, order)
            from sage.matrix.constructor import matrix
            mat = matrix(self.base_ring(), [g._vector_(order=order) for g in elements])
            # Echelonizing a matrix over a field returned the rref
            if row_reduced and self.base_ring() not in Fields():
                try:
                    mat = mat.rref().change_ring(self.base_ring())
                except (ValueError, TypeError):
                    raise ValueError("unable to compute the row reduced echelon form")
            else:
                mat.echelonize()
            ret = [self.from_vector(vec, order=order) for vec in mat if vec]
            return ret

        def invariant_module(self, S, action=operator.mul, action_on_basis=None,
                             side='left', **kwargs):
            r"""
            Return the submodule of ``self`` invariant under the action
            of ``S``.

            For a semigroup `S` acting on a module `M`, the invariant
            submodule is given by

            .. MATH::

                M^S = \{m \in M : s \cdot m = m,\, \forall s \in S\}.

            INPUT:

            - ``S`` -- a finitely-generated semigroup
            - ``action`` -- a function (default: :obj:`operator.mul`)
            - ``side`` -- ``'left'`` or ``'right'`` (default: ``'right'``);
              which side of ``self`` the elements of ``S`` acts
            - ``action_on_basis`` -- (optional) define the action of ``S``
              on the basis of ``self``

            OUTPUT: :class:`~sage.modules.with_basis.invariant.FiniteDimensionalInvariantModule`

            EXAMPLES:

            We build the invariant module of the permutation representation
            of the symmetric group::

                sage: # needs sage.combinat sage.groups sage.modules
                sage: G = SymmetricGroup(3); G.rename('S3')
                sage: M = FreeModule(ZZ, [1,2,3], prefix='M'); M.rename('M')
                sage: action = lambda g, x: M.term(g(x))
                sage: I = M.invariant_module(G, action_on_basis=action); I
                (S3)-invariant submodule of M
                sage: I.basis()
                Finite family {0: B[0]}
                sage: [I.lift(b) for b in I.basis()]
                [M[1] + M[2] + M[3]]
                sage: G.rename(); M.rename()  # reset the names

            We can construct the invariant module of any module that has
            an action of ``S``. In this example, we consider the dihedral
            group `G = D_4` and the subgroup `H < G` of all rotations. We
            construct the `H`-invariant module of the group algebra `\QQ[G]`::

                sage: # needs sage.groups
                sage: G = groups.permutation.Dihedral(4)
                sage: H = G.subgroup(G.gen(0))
                sage: H
                Subgroup generated by [(1,2,3,4)]
                 of (Dihedral group of order 8 as a permutation group)
                sage: H.cardinality()
                4

                sage: # needs sage.groups sage.modules
                sage: A = G.algebra(QQ)
                sage: I = A.invariant_module(H)
                sage: [I.lift(b) for b in I.basis()]
                [() + (1,2,3,4) + (1,3)(2,4) + (1,4,3,2),
                 (2,4) + (1,2)(3,4) + (1,3) + (1,4)(2,3)]
                sage: all(h * I.lift(b) == I.lift(b)
                ....:     for b in I.basis() for h in H)
                True
            """
            if action_on_basis is not None:
                from sage.modules.with_basis.representation import Representation
                M = Representation(S, self, action_on_basis, side=side)
            else:
                M = self

            from sage.modules.with_basis.invariant import FiniteDimensionalInvariantModule
            return FiniteDimensionalInvariantModule(M, S, action=action, side=side, **kwargs)

        def twisted_invariant_module(self, G, chi,
                                     action=operator.mul,
                                     action_on_basis=None,
                                     side='left',
                                     **kwargs):
            r"""
            Create the isotypic component of the action of ``G`` on
            ``self`` with irreducible character given by ``chi``.

            .. SEEALSO::

                -:class:`~sage.modules.with_basis.invariant.FiniteDimensionalTwistedInvariantModule`

            INPUT:

            - ``G`` -- a finitely-generated group
            - ``chi`` -- list/tuple of character values or an instance of
              :class:`~sage.groups.class_function.ClassFunction_gap`
            - ``action`` -- a function (default: :obj:`operator.mul`)
            - ``action_on_basis`` -- (optional) define the action of ``g``
              on the basis of ``self``
            - ``side`` -- ``'left'`` or ``'right'`` (default: ``'right'``);
              which side of ``self`` the elements of ``S`` acts

            OUTPUT: :class:`~sage.modules.with_basis.invariant.FiniteDimensionalTwistedInvariantModule`

            EXAMPLES::

                sage: # needs sage.combinat sage.groups sage.modules
                sage: M = CombinatorialFreeModule(QQ, [1,2,3])
                sage: G = SymmetricGroup(3)
                sage: def action(g, x): return(M.term(g(x)))  # permute coordinates
                sage: T = M.twisted_invariant_module(G, [2,0,-1],
                ....:                                action_on_basis=action)
                sage: import __main__; __main__.action = action
                sage: TestSuite(T).run()
            """

            if action_on_basis is not None:
                from sage.modules.with_basis.representation import Representation
                from sage.categories.modules import Modules
                category = kwargs.pop('category', Modules(self.base_ring()).WithBasis())
                M = Representation(G, self, action_on_basis, side=side, category=category)
            else:
                M = self

            from sage.modules.with_basis.invariant import FiniteDimensionalTwistedInvariantModule
            return FiniteDimensionalTwistedInvariantModule(M, G, chi,
                                                          action, side, **kwargs)

    class ElementMethods:
        def dense_coefficient_list(self, order=None):
            """
            Return a list of *all* coefficients of ``self``.

            By default, this list is ordered in the same way as the
            indexing set of the basis of the parent of ``self``.

            INPUT:

            - ``order`` -- (optional) an ordering of the basis indexing set

            EXAMPLES::

                sage: # needs sage.modules
                sage: v = vector([0, -1, -3])
                sage: v.dense_coefficient_list()
                [0, -1, -3]
                sage: v.dense_coefficient_list([2,1,0])
                [-3, -1, 0]
                sage: sorted(v.coefficients())
                [-3, -1]
            """
            if order is None:
                try:
                    order = sorted(self.parent().basis().keys())
                except AttributeError: # Not a family, assume it is list-like
                    order = range(self.parent().dimension())
            return [self[i] for i in order]

        def _vector_(self, order=None):
            r"""
            Return ``self`` as a vector.

            EXAMPLES::

                sage: # needs sage.modules
                sage: v = vector([0, -1, -3])
                sage: v._vector_()
                (0, -1, -3)
                sage: C = CombinatorialFreeModule(QQ['x'], ['a','b','c'])
                sage: C.an_element()._vector_()
                (2, 2, 3)
            """
            if order is None:
                dense_free_module = self.parent()._dense_free_module()
            else:
                from sage.modules.free_module import FreeModule
                dense_free_module = FreeModule(self.parent().base_ring(), len(order))
            # We slightly break encapsulation for speed reasons
            return dense_free_module.element_class(dense_free_module,
                                                   self.dense_coefficient_list(order),
                                                   coerce=True, copy=False)

    class MorphismMethods:
        def matrix(self, base_ring=None, side='left'):
            r"""
            Return the matrix of this morphism in the distinguished
            bases of the domain and codomain.

            INPUT:

            - ``base_ring`` -- a ring (default: ``None``, meaning the
              base ring of the codomain)

            - ``side`` -- ``'left'`` or ``'right'`` (default: ``'left'``)

            If ``side`` is "left", this morphism is considered as
            acting on the left; i.e. each column of the matrix
            represents the image of an element of the basis of the
            domain.

            The order of the rows and columns matches with the order
            in which the bases are enumerated.

            .. SEEALSO:: :func:`Modules.WithBasis.ParentMethods.module_morphism`

            EXAMPLES::

                sage: # needs sage.modules
                sage: X = CombinatorialFreeModule(ZZ, [1,2]); x = X.basis()
                sage: Y = CombinatorialFreeModule(ZZ, [3,4]); y = Y.basis()
                sage: phi = X.module_morphism(on_basis={1: y[3] + 3*y[4],
                ....:                                   2: 2*y[3] + 5*y[4]}.__getitem__,
                ....:                         codomain=Y)
                sage: phi.matrix()
                [1 2]
                [3 5]
                sage: phi.matrix(side='right')
                [1 3]
                [2 5]

                sage: phi.matrix().parent()                                             # needs sage.modules
                Full MatrixSpace of 2 by 2 dense matrices over Integer Ring
                sage: phi.matrix(QQ).parent()                                           # needs sage.modules
                Full MatrixSpace of 2 by 2 dense matrices over Rational Field

            The resulting matrix is immutable::

                sage: phi.matrix().is_mutable()                                         # needs sage.modules
                False

            The zero morphism has a zero matrix::

                sage: Hom(X, Y).zero().matrix()                                         # needs sage.modules
                [0 0]
                [0 0]

            .. TODO::

                Add support for morphisms where the codomain has a
                different base ring than the domain::

                    sage: Y = CombinatorialFreeModule(QQ, [3,4]); y = Y.basis()         # needs sage.modules
                    sage: phi = X.module_morphism(on_basis={1: y[3] + 3*y[4],           # needs sage.modules
                    ....:                                   2: 2*y[3] + 5/2*y[4]}.__getitem__,
                    ....:                         codomain=Y)
                    sage: phi.matrix().parent()         # not implemented               # needs sage.modules
                    Full MatrixSpace of 2 by 2 dense matrices over Rational Field

                This currently does not work because, in this case,
                the morphism is just in the category of commutative
                additive groups (i.e. the intersection of the
                categories of modules over `\ZZ` and over `\QQ`)::

                    sage: phi.parent().homset_category()                                # needs sage.modules
                    Category of commutative additive semigroups
                    sage: phi.parent().homset_category()        # not implemented, needs sage.modules
                    Category of finite dimensional modules with basis over Integer Ring

            TESTS:

            Check that :issue:`23216` is fixed::

                sage: # needs sage.modules
                sage: X = CombinatorialFreeModule(QQ, [])
                sage: Y = CombinatorialFreeModule(QQ, [1,2,3])
                sage: Hom(X, Y).zero().matrix()
                []
                sage: Hom(X, Y).zero().matrix().parent()
                Full MatrixSpace of 3 by 0 dense matrices over Rational Field
            """
            if base_ring is None:
                base_ring = self.codomain().base_ring()

            on_basis = self.on_basis()
            basis_keys = self.domain().basis().keys()
            from sage.matrix.matrix_space import MatrixSpace
            if isinstance(basis_keys, list):
                nrows = len(basis_keys)
            else:
                nrows = basis_keys.cardinality()
            MS = MatrixSpace(base_ring, nrows, self.codomain().dimension())
            m = MS([on_basis(x)._vector_() for x in basis_keys])
            if side == "left":
                m = m.transpose()
            m.set_immutable()
            return m

        def _repr_matrix(self):
            r"""
            Return a string representation of this morphism (as a matrix).

            EXAMPLES::

                sage: # needs sage.modules
                sage: M = matrix(ZZ, [[1, 0, 0], [0, 1, 0]],
                ....:            column_keys=['a', 'b', 'c'],
                ....:            row_keys=['v', 'w']); M
                Generic morphism:
                From: Free module generated by {'a', 'b', 'c'} over Integer Ring
                To:   Free module generated by {'v', 'w'} over Integer Ring
                sage: M._repr_ = M._repr_matrix
                sage: M                                         # indirect doctest
                  a b c
                v[1 0 0]
                w[0 1 0]
            """
            matrix = self.matrix()

            from sage.matrix.constructor import options

            if matrix.nrows() <= options.max_rows() and matrix.ncols() <= options.max_cols():
                return matrix.str(top_border=self.domain().basis().keys(),
                                  left_border=self.codomain().basis().keys())

            return repr(matrix)

        def _ascii_art_matrix(self):
            r"""
            Return an ASCII art representation of this morphism (as a matrix).

            EXAMPLES::

                sage: # needs sage.modules
                sage: M = matrix(ZZ, [[1, 0, 0], [0, 1, 0]],
                ....:            column_keys=['a', 'b', 'c'],
                ....:            row_keys=['v', 'w']); M
                Generic morphism:
                From: Free module generated by {'a', 'b', 'c'} over Integer Ring
                To:   Free module generated by {'v', 'w'} over Integer Ring
                sage: M._ascii_art_ = M._ascii_art_matrix
                sage: ascii_art(M)                              # indirect doctest
                    a b c
                  v[1 0 0]
                  w[0 1 0]
            """
            matrix = self.matrix()

            from sage.matrix.constructor import options

            if matrix.nrows() <= options.max_rows() and matrix.ncols() <= options.max_cols():
                return matrix.str(character_art=True,
                                  top_border=self.domain().basis().keys(),
                                  left_border=self.codomain().basis().keys())

            from sage.typeset.ascii_art import AsciiArt

            return AsciiArt(repr(self).splitlines())

        def _unicode_art_matrix(self):
            r"""
            Return a unicode art representation of this morphism (as a matrix).

            EXAMPLES::

                sage: # needs sage.modules
                sage: M = matrix(ZZ, [[1, 0, 0], [0, 1, 0]],
                ....:            column_keys=['a', 'b', 'c'],
                ....:            row_keys=['v', 'w']); M
                Generic morphism:
                From: Free module generated by {'a', 'b', 'c'} over Integer Ring
                To:   Free module generated by {'v', 'w'} over Integer Ring
                sage: M._unicode_art_ = M._unicode_art_matrix
                sage: unicode_art(M)                            # indirect doctest
                    a b c
                  v⎛1 0 0⎞
                  w⎝0 1 0⎠
            """
            matrix = self.matrix()

            from sage.matrix.constructor import options

            if matrix.nrows() <= options.max_rows() and matrix.ncols() <= options.max_cols():
                return matrix.str(unicode=True, character_art=True,
                                  top_border=self.domain().basis().keys(),
                                  left_border=self.codomain().basis().keys())

            from sage.typeset.unicode_art import UnicodeArt

            return UnicodeArt(repr(self).splitlines())

        def __invert__(self):
            """
            Return the inverse morphism of ``self``.

            This is achieved by inverting the ``self.matrix()``.
            An error is raised if ``self`` is not invertible.

            EXAMPLES::

                sage: # needs sage.modules
                sage: category = FiniteDimensionalModulesWithBasis(ZZ)
                sage: X = CombinatorialFreeModule(ZZ, [1,2], category=category); X.rename('X'); x = X.basis()
                sage: Y = CombinatorialFreeModule(ZZ, [3,4], category=category); Y.rename('Y'); y = Y.basis()
                sage: phi = X.module_morphism(on_basis={1: y[3] + 3*y[4], 2: 2*y[3] + 5*y[4]}.__getitem__,
                ....:                         codomain=Y, category=category)
                sage: psi = ~phi
                sage: psi
                Generic morphism:
                  From: Y
                  To:   X
                sage: psi.parent()
                Set of Morphisms from Y to X in Category of finite dimensional modules with basis over Integer Ring
                sage: psi(y[3])
                -5*B[1] + 3*B[2]
                sage: psi(y[4])
                2*B[1] - B[2]
                sage: psi.matrix()
                [-5  2]
                [ 3 -1]
                sage: psi(phi(x[1])), psi(phi(x[2]))
                (B[1], B[2])
                sage: phi(psi(y[3])), phi(psi(y[4]))
                (B[3], B[4])

            We check that this function complains if the morphism is not invertible::

                sage: # needs sage.modules
                sage: phi = X.module_morphism(on_basis={1: y[3] + y[4], 2: y[3] + y[4]}.__getitem__,
                ....:                         codomain=Y, category=category)
                sage: ~phi
                Traceback (most recent call last):
                ...
                RuntimeError: morphism is not invertible
                sage: phi = X.module_morphism(on_basis={1: y[3] + y[4], 2: y[3] + 5*y[4]}.__getitem__,
                ....:                         codomain=Y, category=category)
                sage: ~phi
                Traceback (most recent call last):
                ...
                RuntimeError: morphism is not invertible
            """
            mat = self.matrix()
            try:
                inv_mat = mat.parent()(~mat)
            except (ZeroDivisionError, TypeError):
                raise RuntimeError("morphism is not invertible")
            return self.codomain().module_morphism(
                matrix=inv_mat,
                codomain=self.domain(), category=self.category_for())

        def kernel_basis(self):
            """
            Return a basis of the kernel of ``self`` in echelon form.

            EXAMPLES::

                sage: SGA = SymmetricGroupAlgebra(QQ, 3)                                # needs sage.groups sage.modules
                sage: f = SGA.module_morphism(lambda x: SGA(x**2), codomain=SGA)        # needs sage.groups sage.modules
                sage: f.kernel_basis()                                                  # needs sage.groups sage.modules
                ([1, 2, 3] - [3, 2, 1], [1, 3, 2] - [3, 2, 1], [2, 1, 3] - [3, 2, 1])
            """
            return tuple(map( self.domain().from_vector,
                              self.matrix().right_kernel_matrix().rows() ))

        def kernel(self):
            """
            Return the kernel of ``self`` as a submodule of the domain.

            EXAMPLES::

                sage: # needs sage.groups sage.modules
                sage: SGA = SymmetricGroupAlgebra(QQ, 3)
                sage: f = SGA.module_morphism(lambda x: SGA(x**2), codomain=SGA)
                sage: K = f.kernel()
                sage: K
                Free module generated by {0, 1, 2} over Rational Field
                sage: K.ambient()
                Symmetric group algebra of order 3 over Rational Field
            """
            D = self.domain()
            return D.submodule(self.kernel_basis(), already_echelonized=True,
                               category=self.category_for())

        def image_basis(self):
            """
            Return a basis for the image of ``self`` in echelon form.

            EXAMPLES::

                sage: SGA = SymmetricGroupAlgebra(QQ, 3)                                # needs sage.groups sage.modules
                sage: f = SGA.module_morphism(lambda x: SGA(x**2), codomain=SGA)        # needs sage.groups sage.modules
                sage: f.image_basis()                                                   # needs sage.groups sage.modules
                ([1, 2, 3], [2, 3, 1], [3, 1, 2])
            """
            C = self.codomain()
            return tuple(C.echelon_form( map(self, self.domain().basis()) ))

        def image(self):
            """
            Return the image of ``self`` as a submodule of the codomain.

            EXAMPLES::

                sage: SGA = SymmetricGroupAlgebra(QQ, 3)                                # needs sage.groups sage.modules
                sage: f = SGA.module_morphism(lambda x: SGA(x**2), codomain=SGA)        # needs sage.groups sage.modules
                sage: f.image()                                                         # needs sage.groups sage.modules
                Free module generated by {0, 1, 2} over Rational Field
            """
            C = self.codomain()
            return C.submodule(self.image_basis(), already_echelonized=True,
                               category=self.category_for())

    class Homsets(HomsetsCategory):

        class Endset(CategoryWithAxiom):

            class ElementMethods:

                @lazy_attribute
                def characteristic_polynomial(self):
                    r"""
                    Return the characteristic polynomial of this endomorphism.

                    :meth:`characteristic_polynomial` and :meth:`charpoly` are the same method.

                    INPUT:

                    - ``var`` -- variable

                    EXAMPLES::

                        sage: # needs sage.modules
                        sage: V = ZZ^2; phi = V.hom([V.0 + V.1, 2*V.1])
                        sage: phi.characteristic_polynomial()
                        x^2 - 3*x + 2
                        sage: phi.charpoly()
                        x^2 - 3*x + 2
                        sage: phi.matrix().charpoly()
                        x^2 - 3*x + 2
                        sage: phi.charpoly('T')
                        T^2 - 3*T + 2
                        sage: W = CombinatorialFreeModule(ZZ, ['x', 'y'])
                        sage: M = matrix(ZZ, [[1, 0], [1, 2]])
                        sage: psi = W.module_morphism(matrix=M, codomain=W)
                        sage: psi.charpoly()
                        x^2 - 3*x + 2
                    """
                    return self.matrix().charpoly

                charpoly = characteristic_polynomial

                @lazy_attribute
                def determinant(self):
                    """
                    Return the determinant of this endomorphism.

                    :meth:`determinant` and :meth:`det` are the same method.

                    EXAMPLES::

                        sage: # needs sage.modules
                        sage: V = ZZ^2; phi = V.hom([V.0 + V.1, 2*V.1])
                        sage: phi.determinant()
                        2
                        sage: phi.det()
                        2
                        sage: W = CombinatorialFreeModule(ZZ, ['x', 'y'])
                        sage: M = matrix(ZZ, [[1, 0], [1, 2]])
                        sage: psi = W.module_morphism(matrix=M, codomain=W)
                        sage: psi.det()
                        2
                    """
                    return self.matrix().determinant

                det = determinant

                @lazy_attribute
                def fcp(self):
                    """
                    Return the factorization of the characteristic polynomial.

                    INPUT:

                    - ``var`` -- variable

                    EXAMPLES::

                        sage: # needs sage.modules
                        sage: V = ZZ^2; phi = V.hom([V.0 + V.1, 2*V.1])
                        sage: phi.fcp()                                                         # needs sage.libs.pari
                        (x - 2) * (x - 1)
                        sage: phi.fcp('T')                                                      # needs sage.libs.pari
                        (T - 2) * (T - 1)
                        sage: W = CombinatorialFreeModule(ZZ, ['x', 'y'])
                        sage: M = matrix(ZZ, [[1, 0], [1, 2]])
                        sage: psi = W.module_morphism(matrix=M, codomain=W)
                        sage: psi.fcp()                                                         # needs sage.libs.pari
                        (x - 2) * (x - 1)
                    """
                    return self.matrix().fcp

                @lazy_attribute
                def minimal_polynomial(self):
                    r"""
                    Return the minimal polynomial of this endomorphism.

                    :meth:`minimal_polynomial` and :meth:`minpoly` are the same method.

                    INPUT:

                    - ``var`` -- string (default: ``'x'``); a variable name

                    EXAMPLES:

                    Compute the minimal polynomial, and check it. ::

                        sage: # needs sage.modules
                        sage: V = GF(7)^3
                        sage: H = V.Hom(V)([[0,1,2], [-1,0,3], [2,4,1]]); H
                        Vector space morphism represented by the matrix:
                        [0 1 2]
                        [6 0 3]
                        [2 4 1]
                        Domain:   Vector space of dimension 3 over Finite Field of size 7
                        Codomain: Vector space of dimension 3 over Finite Field of size 7
                        sage: H.minpoly()                                                       # needs sage.libs.pari
                        x^3 + 6*x^2 + 6*x + 1
                        sage: H.minimal_polynomial()                                            # needs sage.libs.pari
                        x^3 + 6*x^2 + 6*x + 1
                        sage: H^3 + (H^2)*6 + H*6 + 1
                        Vector space morphism represented by the matrix:
                        [0 0 0]
                        [0 0 0]
                        [0 0 0]
                        Domain:   Vector space of dimension 3 over Finite Field of size 7
                        Codomain: Vector space of dimension 3 over Finite Field of size 7

                        sage: # needs sage.modules sage.rings.finite_rings
                        sage: k = GF(9, 'c')
                        sage: V = CombinatorialFreeModule(k, ['x', 'y', 'z', 'w'])
                        sage: A = matrix(k, 4, [1,1,0,0, 0,1,0,0, 0,0,5,0, 0,0,0,5])
                        sage: phi = V.module_morphism(matrix=A, codomain=V)
                        sage: factor(phi.minpoly())
                        (x + 1) * (x + 2)^2
                        sage: A.minpoly()(A) == 0
                        True
                        sage: factor(phi.charpoly())
                        (x + 1)^2 * (x + 2)^2
                    """
                    return self.matrix().minimal_polynomial

                minpoly = minimal_polynomial

                @lazy_attribute
                def trace(self):
                    r"""
                    Return the trace of this endomorphism.

                    EXAMPLES::

                        sage: # needs sage.modules
                        sage: V = ZZ^2; phi = V.hom([V.0 + V.1, 2*V.1])
                        sage: phi.trace()
                        3
                        sage: W = CombinatorialFreeModule(ZZ, ['x', 'y'])
                        sage: M = matrix(ZZ, [[1, 0], [1, 2]])
                        sage: psi = W.module_morphism(matrix=M, codomain=W)
                        sage: psi.trace()
                        3
                    """
                    return self.matrix().trace

    class TensorProducts(TensorProductsCategory):

        def extra_super_categories(self):
            """
            Implement the fact that a (finite) tensor product of
            finite dimensional modules is a finite dimensional module.

            EXAMPLES::

                sage: C = ModulesWithBasis(ZZ).FiniteDimensional().TensorProducts()
                sage: C.extra_super_categories()
                [Category of finite dimensional modules with basis over Integer Ring]
                sage: C.FiniteDimensional()
                Category of tensor products of
                 finite dimensional modules with basis over Integer Ring
            """
            return [self.base_category()]
